import mvcState
from pygame.sprite import RenderUpdates
from pygame import Rect
from log import log
from objects import Bladder, ManualHeater, Bubble, Cloud, Stripe, Geyser, Drop,Purse, SolarHeater, WaterWheelHeater, WindHeater, FireHeater, NuclearHeater, MoneyWidget

from utils import COLORS
from poutine.statusbar import StatusBar
from random import Random
rng = Random()

class View:
	def ModelStarted( self, model ):
		self.model = model
		self.screen.blit( self.bgImage, (0,0) )
		self.display.flip()

	def Clear( self ):
		for group in self.groups:
			group.clear( self.screen, self.bgImage )

	def Draw( self ):
		changedRects = []
		for group in self.groups:
			changedRects += group.draw(self.screen)
		self.display.update( changedRects )

	def Update( self, timeChange ):
		pass

class GameView(View):
	def __init__( self, screen, display ):
		self.screen = screen
		self.screenRect = screen.get_rect()
		self.display = display
		self.model = None
		self.currentTime = 0

		self.bgImage = load_png( 'bg_game.png' )

		self.hiGroup = RenderUpdates()
		self.lowGroup = RenderUpdates()
		self.viewOnlyGroup = RenderUpdates()

		self.groups = [self.lowGroup, self.viewOnlyGroup, self.hiGroup ]

		self.stripeHeights = [210, 170, 130, 90, 50, 10]
		self.heaterRects = { 
		                    'violet': Rect ( 150, 470, 80, 100 ),
		                    'blue': Rect ( 250, 470, 80, 100 ),
		                    'green': Rect ( 350, 470, 80, 100 ),
		                    'yellow': Rect ( 450, 470, 80, 100 ),
		                    'orange': Rect ( 550, 470, 80, 100 ),
		                    'red': Rect ( 650, 470, 80, 100 ),
		}


		self.purseStatusbars = []

		controller = mvcState.GetController()
		controller.gameEventListeners.append( self )

	def ModelStarted( self, model ):
		View.ModelStarted( self, model )
		heater = self.model.manualHeater
		heater.rect.topleft = (430,478)
		self.hiGroup.add( heater )

		cloud = self.model.cloud
		cloud.rect.topleft = (10,40)
		self.hiGroup.add( cloud )
		
		monWid = self.model.playerMoney
		monWid.rect.topleft = (10,490)
		self.hiGroup.add( monWid )

		bladder = self.model.bladder
		bladder.rect.topleft = (410,378)
		bladder.FixBottom( bladder.rect.midbottom )
		self.hiGroup.add( bladder )

		m = self.model
		stripes = [m.violetstripe, m.bluestripe, m.greenstripe,
		           m.yellowstripe, m.orangestripe, m.redstripe ]
		for i in range( len(stripes) ):
			stripes[i].rect.topleft = ( 400, self.stripeHeights[i] )
			self.lowGroup.add( stripes[i] )

		for i in range( len(m.colorPurses) ):
			cName = m.colorPurses[i].colorName
			dimensions = self.heaterRects[cName].move(0,0)
			dimensions.move_ip(0,100)
			dimensions.height = 10
			sb = StatusBar( m.colorPurses[i], 
			         dimensions,
			         outlineImg='sb_outline.png',
			         attrName='amount', 
			         fullAmt=m.colorPurses[i].capacity,
			         innerColor=COLORS[cName]
			        )
			self.purseStatusbars.append( sb )
			self.lowGroup.add( sb )


	def OnBubbleLaunch( self, centerx ):
		log.debug( 'bubble birth' )
		bubble = Bubble( 438 )
		xpos = int(rng.normalvariate( 0,50 )) + centerx
		bubble.rect.x = xpos
		bubble.rect.bottom = 468
		self.viewOnlyGroup.add( bubble )

	def Update( self, timeChange ):
		self.viewOnlyGroup.update( timeChange )

		self.currentTime += timeChange
		twoSecondsAgo = self.currentTime - 2000
		halfSecondAgo = self.currentTime - 500

		heaterRange = [444,530]

		for sb in self.purseStatusbars:
			sb.update()

	def OnBladderShoot( self, power, height ):
		if height > 3:
			gey = Geyser( "tall" )
		else:
			gey = Geyser( "short" )
		gey.rect.midtop = (450, self.stripeHeights[height])
		self.viewOnlyGroup.add( gey )

	def OnBladderVent( self, power, height ):
		if height > 3:
			gey = Geyser( "tall" )
		else:
			gey = Geyser( "short" )
		gey.rect.midtop = (450, self.stripeHeights[height])
		self.viewOnlyGroup.add( gey )

	def OnDropBirth( self, drop ):
		self.lowGroup.add( drop )

	def OnHeaterBirth( self, heater ):
		switch= { SolarHeater:'violet',
		          WaterWheelHeater:'blue',
		          WindHeater:'green',
		          FireHeater:'yellow',
		          NuclearHeater:'orange',
		}
		klass = heater.__class__
		heater.rect.topleft = self.heaterRects[switch[klass]].topleft
		self.hiGroup.add( heater )


class FreestyleModel:
	def __init__( self ):
		self.time = 0

		self.colorPurses = [
		                    Purse('violet'),
		                    Purse('blue'),
		                    Purse('green'),
		                    Purse('yellow'),
		                    Purse('orange'),
		                    Purse('red'),
		                   ]

		self.objects = RenderUpdates()

		self.playerMoney = MoneyWidget( 0 )
		self.objects.add( self.playerMoney )

		self.manualHeater = ManualHeater()
		self.objects.add( self.manualHeater )

		self.cloud = Cloud()
		self.objects.add( self.cloud )

		self.bladder = Bladder()
		self.objects.add( self.bladder )

		self.violetstripe = Stripe( 'violet', 0 )
		self.objects.add( self.violetstripe )
		self.bluestripe = Stripe( 'blue', 1 )
		self.objects.add( self.bluestripe )
		self.greenstripe = Stripe( 'green', 2 )
		self.objects.add( self.greenstripe )
		self.yellowstripe = Stripe( 'yellow', 3 )
		self.objects.add( self.yellowstripe )
		self.orangestripe = Stripe( 'orange', 4 )
		self.objects.add( self.orangestripe )
		self.redstripe = Stripe( 'red', 5 )
		self.objects.add( self.redstripe )

		self.bonusStripesThisRound = []

	def Update( self, timeChange ):
		self.bonusStripesThisRound = []

		self.time += timeChange
		self.objects.update( timeChange )

		if self.bonusStripesThisRound and not self.cloud.isDripping:
			stripe = rng.choice( self.bonusStripesThisRound )
			colorName = stripe.name
			self.cloud.StartDrip( colorName )

	def Start( self ):
		mvcState.GetView().ModelStarted(self)

		controller = mvcState.GetController()
		controller.mouseListeners.append( self.manualHeater )
		controller.gameEventListeners.append( self )

	def Quit( self ):
		mvcState.SetModel( None )

	def OnStripeAtBonusOpacity( self, stripe ):
		self.bonusStripesThisRound.append( stripe )

	def OnLaunchDrop( self, pos, colorName ):
		drop = Drop( colorName, pos )
		self.objects.add( drop )

	def OnDropOverCapacity( self, colorName ):
		log.debug( 'adding 10 money' )
		self.playerMoney.ChangeMoney( 10 )

	def OnPurseFull( self, colorName ):
		switch= { 'violet': SolarHeater,
		          'blue': WaterWheelHeater,
		          'green': WindHeater,
		          'yellow': FireHeater,
		          'orange': NuclearHeater,
		          'red': None
		}
		klass = switch[colorName]
		if not klass:
			return
		for s in self.objects.sprites():
			if s.__class__ == klass:
				return
		newHeater = klass()
		self.objects.add( newHeater )



#NOTE: the MainMenu is also a View
from utils import load_png
from gui import ImgButton
class MainMenu(View):
	def __init__( self, screen, display ):
		self.nextModelClass = None
		self.screen = screen
		self.screenRect = screen.get_rect()
		self.display = display

		self.model = None

		self.bgImage = load_png( 'bg_mainmenu.png' )

		self.btnGroup = RenderUpdates()

		fBtn = ImgButton( 'freestyle', self.Freestyle )
		fBtn.rect.midtop = self.screenRect.midtop
		fBtn.rect.y += 100
		self.btnGroup.add( fBtn )
		self.freestyleButton = fBtn

		self.groups = [ self.btnGroup ]


	def Update( self, timeChange ):
		self.btnGroup.update()

	def Start( self ):
		controller = mvcState.GetController()
		controller.mouseListeners.append( self.freestyleButton )
		mvcState.GetView().ModelStarted(self)

	def Quit( self ):
		print 'mm quitting'
		controller = mvcState.GetController()
		controller.mouseListeners.remove( self.freestyleButton )
		newModel = self.nextModelClass()
		mvcState.SetModel( newModel )

	def Freestyle( self ):
		self.nextModelClass = FreestyleModel
		self.Quit()
	def Race( self ):
		self.nextModelClass = RaceModel
		self.Quit()
	def Level( self ):
		self.nextModelClass = LevelModel
		self.Quit()

