from log import log
import pygame
import mvcState
from pygame.sprite import Sprite

from random import Random
rng = Random()

from settings import *

class Bladder(Sprite):
	def __init__(self):
		Sprite.__init__(self)
		self.fillingImgs = [
		                    load_png( 'bladder00.png' ),
		                    load_png( 'bladder01.png' ),
		                    load_png( 'bladder02.png' ),
		                    load_png( 'bladder03.png' ),
		                    load_png( 'bladder04.png' ),
		                   ]
		self.tooFullImg = load_png( 'bladder_toofull.png' )
		self.ventImg = self.tooFullImg
		self.shootImg = load_png( 'bladder_shoot.png' )
		self.image = self.fillingImgs[0]
		self.rect = self.image.get_rect()

		self.originalMidBottom = self.rect.midbottom
		self.dirty = 1

		self.capacity = 10
		self.numBubbles = 0
		self.isShooting = False
		self.isVenting = False
		self.overCapacityCounter = 0
		self.tooFullAnimCounter= 0
		self.squeezeFactor = 0
		self.maxPower = 3

		self.tooFullAnim = [ (2,0), (-2,0) ]
		self.tooFullAnimIndex = 0

		controller = mvcState.GetController()
		controller.gameEventListeners.append( self )
		controller.mouseListeners.append( self )

	def kill( self ):
		Sprite.kill( self )
		controller = mvcState.GetController()
		controller.gameEventListeners.remove( self )

	def FixBottom( self, fixedBottom ):
		if type(fixedBottom[0]) != int or type(fixedBottom[1]) != int:
			raise Exception( "Fix Bottom Failure" )
		self.originalMidBottom = fixedBottom

	def update( self, timeChange ): 
		if not self.dirty:
			return

		if self.isVenting:
			log.debug( 'bladder is venting' )
			controller = mvcState.GetController()
			self.image = self.ventImg
			self.rect = self.image.get_rect()
			self.rect.midbottom = self.originalMidBottom

			power = min( self.numBubbles, self.maxPower )
			height = rng.randint(0,5)
			controller.GameEvent( 'BladderVent', power,height )
			self.numBubbles -= power

			if self.numBubbles < 1:
				self.isVenting = False
				controller.GameEvent( 'BladderVentStop' )

			self.dirty = 1

		elif self.isShooting:
			log.debug( 'bladder is shooting' )
			controller = mvcState.GetController()
			self.image = self.shootImg
			self.rect = self.image.get_rect()
			self.rect.midbottom = self.originalMidBottom

			power = min( self.numBubbles, self.maxPower )
			height = self.squeezeFactor
			controller.GameEvent( 'BladderShoot', power,height )
			self.numBubbles -= power

			if self.numBubbles < 1:
				self.isShooting = False
				controller.GameEvent( 'BladderShootStop' )

			self.dirty = 1

		elif not self.numBubbles > self.capacity:
			fullFraction = float(self.numBubbles) / self.capacity
			#log.debug( 'fullfraction'+str(fullFraction) )
			index = min( int( len(self.fillingImgs)*fullFraction ),
			             len(self.fillingImgs)-1 )
			#log.debug( 'index'+str(index) )
			self.image = self.fillingImgs[index]
			self.rect = self.image.get_rect()
			self.rect.midbottom = self.originalMidBottom

			self.dirty = 0

		else: #bubbles > capacity
			self.overCapacityCounter += timeChange
			self.tooFullAnimCounter += timeChange
			if self.overCapacityCounter > 5000:
				self.isVenting = True
				self.overCapacityCounter = 0
			else:
				self.TooFullAnimate()

			self.dirty = 1

	def TooFullAnimate(self):
		if self.tooFullAnimCounter < 200:
			return
		self.tooFullAnimCounter = 0

		if self.image == self.tooFullImg:
			img = self.fillingImgs[len(self.fillingImgs)-1]
		else:
			img = self.tooFullImg
		self.image = img
		self.rect = self.image.get_rect()
		self.rect.midbottom = self.originalMidBottom

		self.rect.move_ip( *self.tooFullAnim[self.tooFullAnimIndex] )

		self.tooFullAnimIndex += 1
		self.tooFullAnimIndex %= len(self.tooFullAnim)


	def OnBubbleHitCeiling(self):
		self.numBubbles += 1
		self.dirty = 1

	def OnMouseMove( self, event ): 
		if not self.isShooting:
			return
		distFromCenter = min( abs(event.pos[0]-self.rect.centerx), 199 )
		self.squeezeFactor = int( distFromCenter*(6.0/200) )
		#log.debug( 'set squeezefactor to '+ str( self.squeezeFactor ) )
		self.dirty = 1
			


	def OnMouseDown( self, pos ): 
		if not self.rect.collidepoint( pos ):
			return
		if self.numBubbles < 1:
			return
		self.squeezeFactor = 0
		self.isShooting = True
		controller = mvcState.GetController()
		controller.GameEvent( 'BladderShootStart' )
		self.dirty = 1

	def OnMouseUp( self, pos ): 
		if self.isShooting:
			self.isShooting = False
			controller = mvcState.GetController()
			controller.GameEvent( 'BladderShootStop' )
			self.dirty = 1
	


import precomputed_circle
from utils import load_png

class Bubble(Sprite):
	def __init__(self, ceilingY):
		Sprite.__init__(self)
		self.formingImg= load_png( 'bubble_forming.png' )
		self.risingImg= load_png( 'bubble_rising.png' )
		self.rect = self.formingImg.get_rect()
		self.image = self.formingImg

		self.ceilingY = ceilingY
		self.isAtCeiling = False
		self.birthTimeCounter = 0
		self.nextJiggleCounter = 500
		self.deathCounter = 0

	def Rise( self ):
		#log.debug( 'bubble starts rising' )
		self.image = self.risingImg
		tmpRect = self.risingImg.get_rect()
		tmpRect.midbottom = self.rect.midbottom
		self.rect = tmpRect

	def update( self, timeChange ): 
		self.birthTimeCounter += timeChange

		if self.birthTimeCounter > 2000 \
		  and self.image == self.formingImg:
			self.Rise()

		elif self.image == self.risingImg and not self.isAtCeiling:
			self.rect.y = self.rect.y-1
			if self.rect.top < self.ceilingY:
				controller = mvcState.GetController()
				controller.GameEvent( "BubbleHitCeiling" )
				self.isAtCeiling = True
				self.rect.top = self.ceilingY
				self.deathCounter = self.birthTimeCounter + 2000

		if self.birthTimeCounter > self.nextJiggleCounter:
			self.rect.x += rng.randint(-1,1)
			self.nextJiggleCounter += 500

		if self.isAtCeiling \
		  and self.birthTimeCounter > self.deathCounter:
			#log.debug( 'killing bubble' )
		  	self.kill()

class Cloud(Sprite):
	def __init__(self):
		Sprite.__init__(self)
		self.stableImg = load_png( 'cloud.png' )
		self.image = self.stableImg
		self.rect = self.image.get_rect()

		self.jiggleCounter = 1500
		self.jiggles = [ (4,0), (-4,0), (-1,3), (-2,-4), (3,1) ]
		self.jiggleIndex = 0

		self.dripMoves = [ (0,3), (0,2), (0,2), 
		                   (0,1), (0,1), (0,1), (0,1), 
		                   (0,-3), 
		                   (0,-3), 
		                   (0,-3), 
		                   (0,-1), 
		                   (0,-1), 
		                 ]
		self.dripIndex = 0
		self.isDripping = False
		self.dropColor = None

	def Jiggle( self ): 
		self.jiggleCounter = 1500
		self.rect.move_ip( self.jiggles[self.jiggleIndex] )
		self.jiggleIndex += 1
		self.jiggleIndex %= len( self.jiggles )

	def StartDrip( self, colorName ): 
		self.isDripping = True
		self.dropColor = colorName

	def Drip( self ): 
		self.rect.move_ip( self.dripMoves[self.dripIndex] )
		self.dripIndex += 1
		self.dripIndex %= len( self.dripMoves )
		if self.dripIndex == 7:
			controller = mvcState.GetController()
			controller.GameEvent( 'LaunchDrop', self.rect.center,
			                                    self.dropColor )
		if self.dripIndex == 0:
			self.isDripping = False

	def update( self, timeChange ): 
		if not self.isDripping:
			self.jiggleCounter -= timeChange
			if self.jiggleCounter < 0:
				self.Jiggle()

		else:
			self.Drip()

from utils import change_alpha, change_alpha_mult

class Stripe(Sprite):
	def __init__(self, name, height):
		Sprite.__init__(self)

		self.height = height
		self.name = name
		self._opacity= 20
		self.hiOpacityLimit = 255
		self.bonusOpacityLimit = 50
		self.lowOpacityLimit = 20

		self.fullImg= load_png( 'stripe_'+name+'_full.png' )
		self.emptyImg = load_png( 'stripe_'+name+'_empty.png' )
		self.rect = self.emptyImg.get_rect()
		self.ComposeImg()

		self.lastChangeCounter = 0

		controller = mvcState.GetController()
		controller.gameEventListeners.append( self )

		self.dirty = 1

	def OpacityChange( self, change ): 
		log.debug( self.name + 'stripe changes opacity'+ str(change) +str(self._opacity) )
		controller = mvcState.GetController()
		self._opacity += change
		if self._opacity > self.hiOpacityLimit:
			self._opacity = self.hiOpacityLimit
			controller.GameEvent( 'StripeHitMaxOpacity', self )
		if self._opacity < self.lowOpacityLimit:
			self._opacity = self.lowOpacityLimit
		if self._opacity > self.bonusOpacityLimit:
			controller.GameEvent( 'StripeAtBonusOpacity', self )
		self.dirty = 1

	def ComposeImg( self ): 
		self.image = self.fullImg.convert_alpha()
		change_alpha( self.image, self._opacity )
		
	def update( self, timeChange ): 
		self.lastChangeCounter += timeChange
		if self.dirty:
			self.ComposeImg()
			self.lastChangeCounter = 0
			self.dirty = 0

		elif self.lastChangeCounter > 6000:
			self.lastChangeCounter = 0
			self.OpacityChange( -1 )
			self.ComposeImg()

	def OnBladderShoot(self, power, height):
		if height != self.height:
			return
		self.OpacityChange( power )

	def OnBladderVent(self, power, height):
		if height != self.height:
			return
		self.OpacityChange( power )

class Geyser(Sprite):
	def __init__(self, height="tall"):
		Sprite.__init__(self)

		if height == "tall":
			self.image = load_png( 'geyser_tall.png' )
		else:
			self.image = load_png( 'geyser_short.png' )
		self.rect = self.image.get_rect()

		self.jiggleCounter = 500
		self.lifeCounter = 3000

	def Jiggle( self ): 
		self.jiggleCounter = 500
		self.rect.x += rng.randint(-1,1)

	def update( self, timeChange ): 
		self.jiggleCounter -= timeChange
		self.lifeCounter -= timeChange
		if self.jiggleCounter < 0:
			self.Jiggle()
		if self.lifeCounter < 0:
			self.kill()


class Drop(Sprite):
	def __init__(self, colName, startPos):
		Sprite.__init__(self)

		self.colorName = colName
		self.image = load_png( 'drop_'+self.colorName+'.png' )
		self.rect = self.image.get_rect()
		self.rect.midtop = startPos

		controller = mvcState.GetController()
		controller.GameEvent( 'DropBirth', self )

	def update( self, timeChange ): 
		bucketY = 300

		self.rect.move_ip( 0,4 )

		if self.rect.bottom >= bucketY:
			controller = mvcState.GetController()
			controller.GameEvent( 'DropHitsBucket', self.colorName )
			self.kill()

class Purse:
	def __init__(self, colorName):
		self.amount = 0
		self.capacity = 100 
		#self.capacity = 20 #for DEBUG
		self.colorName = colorName

		controller = mvcState.GetController()
		controller.gameEventListeners.append( self )

	def OnDropHitsBucket( self, colorName ):
		if colorName != self.colorName:
			return

		self.amount += 5
		log.debug( 'drop hit the bucket'+ colorName )
		controller = mvcState.GetController()
		if self.amount == self.capacity:
			controller.GameEvent( "PurseFull", colorName )
		elif self.amount > self.capacity:
			controller.GameEvent( "DropOverCapacity", colorName )
			self.amount = self.capacity

	def OnBoughtSolar( self ):
		if self.colorName == 'violet':
			self.amount = 0


class Heater:
	def __init__(self):
		self.temperature = 20
		self.timeSinceBubbleLaunch = 0
	def BubbleLaunch( self ):
		self.timeSinceBubbleLaunch = 0
		controller = mvcState.GetController()
		controller.GameEvent('BubbleLaunch', self.rect.centerx )
	def update( self, timeChange ): 
		self.timeSinceBubbleLaunch += timeChange
		if self.temperature > 190 \
		  and self.timeSinceBubbleLaunch >  bubbleRate190Degrees:
			self.BubbleLaunch()
		elif self.temperature > 100 \
		  and self.timeSinceBubbleLaunch > bubbleRate100Degrees:
			self.BubbleLaunch()

class ManualHeater(Sprite, Heater):
	def __init__(self):
		Sprite.__init__(self)
		Heater.__init__(self)

		self.hiTempLimit = 200
		self.lowTempLimit = 20
		self.isHeating = 0

		self.cursorIndex = 0
		self.xlens = precomputed_circle.xlens[:]
		self.ylens = precomputed_circle.ylens[:]

		self.redImg= load_png( 'heater_red.png' )
		self.blackImg = load_png( 'heater_black.png' )
		self.cursorImg = load_png( 'heater_cursor.png' )
		self.cursorRect = self.cursorImg.get_rect()
		self.rect = self.blackImg.get_rect()
		self.ComposeImg()


		self.dirty = 1

	def ComposeImg( self ): 
		self.image = self.blackImg.convert_alpha()
		tRange = self.hiTempLimit - self.lowTempLimit
		tempPercent = float(self.temperature-self.lowTempLimit)/tRange
		redImg = self.redImg.convert_alpha()
		change_alpha_mult( redImg, tempPercent )

		self.image.blit( redImg, (0,0) )

		x = self.xlens[self.cursorIndex]
		y = self.ylens[self.cursorIndex]
		self.cursorRect.center = ( self.rect.width/2 + x,
		                           self.rect.height/2 + y )
		self.image.blit( self.cursorImg, self.cursorRect.topleft )


	def update( self, timeChange ): 
		Heater.update( self, timeChange )
		self.timeSinceBubbleLaunch += timeChange

		if self.dirty:
			self.ComposeImg()

		if not self.isHeating and self.temperature > self.lowTempLimit:
			self.temperature -= 1

		self.isHeating = 0



	def BubbleLaunch( self ):
		self.timeSinceBubbleLaunch = 0
		controller = mvcState.GetController()
		controller.GameEvent('BubbleLaunch', self.rect.centerx )

	def MouseInCursor( self ):
		self.cursorIndex += 1
		self.cursorIndex %= len( self.xlens )
		self.isHeating = 1
		self.dirty = 1
		self.temperature += 5
		if self.temperature > self.hiTempLimit:
			self.temperature = self.hiTempLimit
		#log.debug( self.temperature )

	def OnMouseMove( self, event ): 
		translatedPos = (event.pos[0] - self.rect.x,
		                event.pos[1] - self.rect.y)
		if self.cursorRect.collidepoint( translatedPos ):
			self.MouseInCursor()


	def OnMouseDown( self, pos ): 
		pass
	def OnMouseUp( self, pos ): 
		pass


class SolarHeater(Sprite, Heater):
	def __init__(self):
		Sprite.__init__(self)
		Heater.__init__(self)

		self.temperature = 101
		self.isActive = False
		self.isClicking = False

		self.offImg   = load_png( 'solar_off.png' )
		self.onImg    = load_png( 'solar_on.png' )
		self.focusImg = load_png( 'solar_focus.png' )
		self.image = self.offImg
		self.rect = self.image.get_rect()

		controller = mvcState.GetController()
		controller.mouseListeners.append( self )
		controller.GameEvent( 'HeaterBirth', self )

	def update( self, timeChange ): 
		if self.isActive:
			Heater.update( self, timeChange )

	def OnMouseMove( self, event ): 
		if self.isActive:
			return

		pos = event.pos
		if self.image == self.offImg and self.rect.collidepoint( pos ):
			log.debug( 'setting focus img' )
			self.image = self.focusImg
		elif self.image == self.focusImg \
		  and not self.rect.collidepoint( pos ):
			log.debug( 'setting off img' )
			self.image = self.offImg

	def OnMouseDown( self, pos ): 
		if self.rect.collidepoint( pos ):
			self.isClicking = True
	def OnMouseUp( self, pos ): 
		if self.rect.collidepoint( pos ) and self.isClicking:
			self.Click()
	def Click( self ): 
		self.image = self.onImg
		self.isActive = True
		controller = mvcState.GetController()
		controller.GameEvent( 'BoughtSolar' )

class FireHeater(Sprite, Heater):
	def __init__(self):
		Sprite.__init__(self)
		Heater.__init__(self)
class WaterWheelHeater(Sprite, Heater):
	def __init__(self):
		Sprite.__init__(self)
		Heater.__init__(self)
class WindHeater(Sprite, Heater):
	def __init__(self):
		Sprite.__init__(self)
		Heater.__init__(self)
class NuclearHeater(Sprite, Heater):
	def __init__(self):
		Sprite.__init__(self)
		Heater.__init__(self)

from pygame import Surface, SRCALPHA
class MoneyWidget(Sprite):
	def __init__(self, amount):
		Sprite.__init__(self)
		self.amount = amount
		self.capacity = 999

		self.dollarImg = load_png( 'alpha_dollar.png' )
		self.imgNums = [
		                load_png( 'alpha_0.png' ),
		                load_png( 'alpha_1.png' ),
		                load_png( 'alpha_2.png' ),
		                load_png( 'alpha_3.png' ),
		                load_png( 'alpha_4.png' ),
		                load_png( 'alpha_5.png' ),
		                load_png( 'alpha_6.png' ),
		                load_png( 'alpha_7.png' ),
		                load_png( 'alpha_8.png' ),
		                load_png( 'alpha_9.png' ),
		               ]

		self.image = self.dollarImg.convert_alpha() #copy it
		self.rect = self.image.get_rect()

		self.ComposeImg()
		self.dirty = 1

	def ComposeImg( self ): 
		strNum = str( self.amount )
		self.image = Surface( (28*(2+len(strNum)), 28), SRCALPHA, 32 )
		self.image.fill( (0,0,0,0) )
		
		x = 0
		self.image.blit( self.dollarImg, (x,0) )
		for char in strNum:
			x += 24
			index = int( char )
			self.image.blit( self.imgNums[index], (x,0) )

		self.rect.width = self.image.get_rect().width

	def update( self, timeChange ): 
		if self.dirty:
			self.ComposeImg()

	def ChangeMoney( self, change ): 
		self.amount += change
		if self.amount > self.capacity:
			self.amount = self.capacity
		self.dirty = 1
